# This file has been generated


build_system = 'AppVeyor'
build_id = '29367322'
build_number = '402'
build_version = '1.0.402'
job_id = 't6aq8tvtvhs78aus'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/stlib/build/job/t6aq8tvtvhs78aus'
